from django.db import models

from unifier.models import UnifiedFrame, UnifiedFrameArgument


class MainVerticalRelation(models.Model):
    name = models.CharField(max_length=100)
    priority = models.PositiveIntegerField()

    class Meta:
        ordering = ['priority']

    def __str__(self):
        return self.name


class TemporalVerticalRelation(models.Model):
    name = models.CharField(max_length=100)
    priority = models.PositiveIntegerField()

    class Meta:
        ordering = ['priority']

    def __str__(self):
        return self.name


class UnifiedFrameArgument2UnifiedFrameArgumentMapping(object):
    pass


class UnifiedFrameVerticalRelation(models.Model):
    source_unified_frame = models.ForeignKey(UnifiedFrame, related_name='source_vertical_relation',
                                             default=None, blank=True, null=True, on_delete=models.PROTECT)
    target_unified_frame = models.ForeignKey(UnifiedFrame, related_name='target_vertical_relation',
                                             default=None, blank=True, null=True, on_delete=models.PROTECT)
    main_vertical_relation = models.ForeignKey(MainVerticalRelation, related_name='unified_frame_vertical_relation',
                                               default=None, blank=True, null=True, on_delete=models.PROTECT)
    temporal_vertical_relation = models.ManyToManyField(related_name='unified_frame_vertical_relation',
                                                        to=TemporalVerticalRelation)
    is_negated = models.BooleanField(default=False)

    iterated_target_unified_frame = models.ForeignKey(UnifiedFrame, related_name='iterated_target_vertical_relation',
                                                      default=None, blank=True, null=True, on_delete=models.PROTECT)
    iterated_temporal_vertical_relation = models.ManyToManyField(related_name='iterated_unified_frame_vertical_relation',
                                                                 to=TemporalVerticalRelation)

    def __str__(self):
        iter_str = ('(iter:' + self.iterated_target_unified_frame.ttile + ' - ' +
         (','.join(self.iterated_temporal_vertical_relation.all()) if self.iterated_temporal_vertical_relation else '') + ')') if self.iterated_target_unified_frame else ''

        return self.source_unified_frame.title + ':' + (self.target_unified_frame.title if self.target_unified_frame else '') + \
               ' - ' + ('[neg]' if self.is_negated else '') + self.main_vertical_relation.name + ', ' + \
               (','.join(self.temporal_vertical_relation.all()) if self.is_negated else '') + iter_str


class UnifiedFrameArgument2UnifiedFrameArgumentMapping(models.Model):
    source_unified_agrument = models.ForeignKey(UnifiedFrameArgument, related_name='source_unified_agrument_mapping',
                                                on_delete=models.PROTECT)
    target_unified_agrument = models.ForeignKey(UnifiedFrameArgument, related_name='target_unified_agrument_mapping', on_delete=models.PROTECT)
    unified_frame_vertical_relation = models.ForeignKey(UnifiedFrameVerticalRelation,
                                                        related_name='unified_frame_arguments_mapping', on_delete=models.PROTECT)

